//  NavLogReader.java
//  (c) Jeffrey Feldstein 2004
//  www.sherpas.com. See copyright notice in Copyright.txt.


import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;


public class NavLogReader
  extends DefaultHandler 
{
	public static boolean endDocument = false;
	protected ModelTest m_xdeTester;
	
	public NavLogReader(ModelTest instance)
 	{
		// super();
		m_xdeTester = instance;
	}	
	private interface ParserStates 
	{
		int
            LOG_TYPE = 0,
            STATE_CHANGE = 1,
            TRANSITION_TAG = 2,
            TRANSITION_VALUE = 3,
            END_TRANSITION_TAG = 4,
            END_STATE_CHANGE = 5;

 	}

	private interface xmlTAGS 
	{
		String
            NAVIGATION_LOG = "NavigationLog",
            CHANGE_STATE = "changeState",
            TRANSITION = "transition";
 	}

	int parserState;
	int transIdx;
	
  public void parseURI(String uri)
  {
    try
    { 
      SAXParserFactory spf = SAXParserFactory.newInstance();
      SAXParser sp = spf.newSAXParser();
      sp.parse(uri, this);
    }
    catch (Exception e)
    {
      System.err.println(e);
      e.printStackTrace(); 
    }
  }
    
  /** Start document. */
  public void startDocument()
  {
  } // startDocument()

  /** Start element. */
  public void startElement(String namespaceURI, String localName, 
                           String rawName, Attributes attrs) 
  {
  	String upperRaw;
  	
  	upperRaw = rawName.toUpperCase();
  	switch (parserState)
  	{
  		case ParserStates.LOG_TYPE:
  			if (rawName.equalsIgnoreCase(xmlTAGS.NAVIGATION_LOG))
  				parserState = ParserStates.STATE_CHANGE;
  			else
     			System.out.print("Bad logType: Looking for: " + xmlTAGS.NAVIGATION_LOG + "\n" +
     			                 "found:                    " + rawName); 			
  			break;
  		
  		case ParserStates.STATE_CHANGE:
  			if (rawName == xmlTAGS.CHANGE_STATE)
  				parserState = ParserStates.TRANSITION_TAG;
  			else
     			System.out.print("Bad State Change: "+ rawName); 			
  			break;

  		case ParserStates.TRANSITION_TAG:
  			if (rawName == xmlTAGS.TRANSITION)
  			{
  				parserState = ParserStates.TRANSITION_VALUE;
 			}
  			else
     			System.out.print("Bad Transition: "+ rawName); 			
  			break;
  	}
  	/*
    System.out.print("<");
    System.out.print(rawName);
    if (attrs != null)
    {
      int len = attrs.getLength();
      for (int i = 0; i < len; i++)
      {
        System.out.print(" ");
        System.out.print(attrs.getQName(i));
        System.out.print("=\"");
        System.out.print(attrs.getValue(i));
        System.out.print("\"");
      }
    }
    System.out.print(">");
    */
  } // startElement(String,AttributeList)

  /** Characters. */
  public void characters(char ch[], int start, int length) 
  {
  	System.out.println("chars parserState: "+ parserState);
	switch (parserState)
  	{
  		case ParserStates.TRANSITION_VALUE:
  			String valueString = new String(ch, start, length);
  			transIdx = Integer.valueOf(valueString).intValue();		
 			System.out.println("transIdx: "+ transIdx);
 			parserState = ParserStates.END_TRANSITION_TAG;  	
 			m_xdeTester.PerformTransition(transIdx);
/* 					
			synchronized(ModelTest.class) {
				System.out.println("Calling notify.");
	            	ModelTest.class.notifyAll();
				System.out.println("notifyied.");
					  			}
*/
 			break;
  	}
    // System.out.print(new String(ch, start, length));
  } // characters(char[],int,int);

  /** Ignorable whitespace. */
  public void ignorableWhitespace(char ch[], int start, int length) 
  {
    characters(ch, start, length);
  } // ignorableWhitespace(char[],int,int);

  /** End element. */
  public void endElement(String namespaceURI, String localName, 
                         String rawName) 
  {
	switch (parserState)
  	{

		case ParserStates.END_STATE_CHANGE:
  			if (rawName == xmlTAGS.CHANGE_STATE)
  				parserState = ParserStates.STATE_CHANGE;
  			else
     			System.out.print("Bad END State Change: "+ rawName); 			
  			break;

  		case ParserStates.END_TRANSITION_TAG:
  			if (rawName == xmlTAGS.TRANSITION)
  			{
  				parserState = ParserStates.END_STATE_CHANGE;
  			}
  			else
     			System.out.print("Bad END Transition: "+ rawName); 			
  			break;
  	}

       //    System.out.print("</");
       //    System.out.print(rawName);
       //    System.out.print(">");
  } // endElement(String)

  /** End document. */
  public void endDocument() 
  {
    // No need to do anything.
    endDocument = true;
  } // endDocument()

  /** Processing instruction. */
  public void processingInstruction(String target, String data) 
  {
    System.out.print("<?");
    System.out.print(target);
    if (data != null && data.length() > 0)
    {
      System.out.print(' ');
      System.out.print(data);
    }
    System.out.print("?>");

  } // processingInstruction(String,String)

  //
  // ErrorHandler methods
  //

  /** Warning. */
  public void warning(SAXParseException ex) 
  {
    System.err.println("[Warning] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
  }

  /** Error. */
  public void error(SAXParseException ex) 
  {
    System.err.println("[Error] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
  }

  /** Fatal error. */
  public void fatalError(SAXParseException ex) 
    throws SAXException 
  {
    System.err.println("[Fatal Error] "+
                       getLocationString(ex)+": "+
                       ex.getMessage());
    throw ex;
  }

  /** Returns a string of the location. */
  private String getLocationString(SAXParseException ex) 
  {
    StringBuffer str = new StringBuffer();

    String systemId = ex.getSystemId();
    if (systemId != null)
    {
      int index = systemId.lastIndexOf('/');
      if (index != -1)
        systemId = systemId.substring(index + 1);
      str.append(systemId);
    }
    str.append(':');
    str.append(ex.getLineNumber());
    str.append(':');
    str.append(ex.getColumnNumber());

    return str.toString();
  } // getLocationString(SAXParseException):String
}